@@ -20,15 +20,10 @@ import sys
 import time
 from contextlib import contextmanager
 
-from tools import cache
-from tools import shared
-from tools import system_libs
-from tools import ports
-from tools import utils
+from tools import cache, ports, shared, system_libs, utils
 from tools.settings import settings
 from tools.system_libs import USE_NINJA
 
-
 # Minimal subset of targets used by CI systems to build enough to be useful
 MINIMAL_TASKS = [
     'libcompiler_rt',
@@ -100,8 +95,6 @@ MINIMAL_TASKS = [
     'libunwind-legacyexcept',
     'libunwind-wasmexcept',
     'libnoexit',
-    'libwebgpu',
-    'libwebgpu_cpp',
     'bullet',
 ]
 
@@ -134,6 +127,9 @@ MINIMAL_PIC_TASKS = MINIMAL_TASKS + [
     'libwasmfs-debug',
     'libwasmfs_no_fs',
     'giflib',
+    'sdl2',
+    'sdl2_gfx',
+    'sdl3',
 ]
 
 PORTS = sorted(list(ports.ports_by_name.keys()) + list(ports.port_variants.keys()))
