@@ -10,16 +10,15 @@ This script acts as a frontend replacement for clang-scan-deps.
 """
 
 import sys
-import emcc
-from tools import shared, cmdline
+
+from tools import cmdline, compile, shared
 
 argv = sys.argv[1:]
 
 # Parse and discard any emcc-specific flags (e.g. -sXXX).
-newargs = cmdline.parse_args(argv)[3]
-newargs = cmdline.parse_s_args(newargs)[1]
+newargs = cmdline.parse_arguments(argv)[1]
 
 # Add any clang flags that emcc would add.
-newargs += emcc.get_cflags(tuple(argv))
+newargs += compile.get_cflags(tuple(argv))
 
 shared.exec_process([shared.CLANG_SCAN_DEPS] + newargs)
