@@ -3,51 +3,27 @@
 // University of Illinois/NCSA Open Source License.  Both these licenses can be
 // found in the LICENSE file.
 
-/*global addToLibrary*/
-
-/*global Module, asm*/
-/*global _malloc, _free, _memcpy*/
-/*global FUNCTION_TABLE, HEAP8, HEAPU8, HEAP16, HEAPU16, HEAP32, HEAPU32, HEAPF32, HEAPF64*/
-/*global AsciiToString*/
-/*global Emval, emval_handle_array, __emval_decref*/
-/*jslint sub:true*/ /* The symbols 'fromWireType' and 'toWireType' must be accessed via array notation to be closure-safe since craftInvokerFunction crafts functions as strings that can't be closured. */
-
-// -- jshint doesn't understand library syntax, so we need to specifically tell it about the symbols we define
-/*global typeDependencies, flushPendingDeletes, getTypeName, getBasestPointer, throwBindingError, UnboundTypeError, embindRepr, registeredInstances, registeredTypes*/
-/*global ensureOverloadTable, embind__requireFunction, awaitingDependencies, makeLegalFunctionName, embind_charCodes:true, registerType, createNamedFunction, RegisteredPointer, throwInternalError*/
-/*global floatReadValueFromPointer, integerReadValueFromPointer, enumReadValueFromPointer, replacePublicSymbol, craftInvokerFunction, tupleRegistrations*/
-/*global finalizationRegistry, attachFinalizer, detachFinalizer, releaseClassHandle, runDestructor*/
-/*global ClassHandle, makeClassHandle, structRegistrations, whenDependentTypesAreResolved, BindingError, deletionQueue, delayFunction:true, upcastPointer*/
-/*global exposePublicSymbol, heap32VectorToArray, char_0, char_9*/
-/*global getInheritedInstanceCount, getLiveInheritedInstances, setDelayFunction, InternalError, runDestructors*/
-/*global requireRegisteredType, unregisterInheritedInstance, registerInheritedInstance, PureVirtualError, throwUnboundTypeError*/
-/*global assert, validateThis, downcastPointer, registeredPointers, RegisteredClass, getInheritedInstance */
-/*global throwInstanceAlreadyDeleted, shallowCopyInternalPointer*/
-/*global RegisteredPointer_fromWireType, constNoSmartPtrRawPointerToWireType, nonConstNoSmartPtrRawPointerToWireType, genericPointerToWireType*/
-
 #include "libembind_shared.js"
 
 var LibraryEmbind = {
   $UnboundTypeError: class extends Error {},
   $PureVirtualError: class extends Error {},
-  $GenericWireTypeSize: {{{ 2 * POINTER_SIZE }}},
 #if EMBIND_AOT
   $InvokerFunctions: '<<< EMBIND_AOT_INVOKERS >>>',
 #endif
   // If register_type is used, emval will be registered multiple times for
   // different type id's, but only a single type object is needed on the JS side
   // for all of them. Store the type for reuse.
-  $EmValType__deps: ['_emval_decref', '$Emval', '$readPointer', '$GenericWireTypeSize'],
+  $EmValType__deps: ['_emval_decref', '$Emval', '$readPointer'],
   $EmValType: `{
     name: 'emscripten::val',
-    'fromWireType': (handle) => {
+    fromWireType: (handle) => {
       var rv = Emval.toValue(handle);
       __emval_decref(handle);
       return rv;
     },
-    'toWireType': (destructors, value) => Emval.toHandle(value),
-    argPackAdvance: GenericWireTypeSize,
-    'readValueFromPointer': readPointer,
+    toWireType: (destructors, value) => Emval.toHandle(value),
+    readValueFromPointer: readPointer,
     destructorFunction: null, // This type does not need a destructor
 
     // TODO: do we need a deleteObject here?  write a test where
@@ -146,8 +122,6 @@ var LibraryEmbind = {
     }
   },
 
-  $createNamedFunction: (name, func) => Object.defineProperty(func, 'name', { value: name }),
-
   $embindRepr: (v) => {
     if (v === null) {
         return 'null';
@@ -221,11 +195,6 @@ var LibraryEmbind = {
   $registerType__deps: ['$sharedRegisterType'],
   $registerType__docs: '/** @param {Object=} options */',
   $registerType: function(rawType, registeredInstance, options = {}) {
-#if ASSERTIONS
-    if (registeredInstance.argPackAdvance === undefined) {
-      throw new TypeError('registerType registeredInstance requires argPackAdvance');
-    }
-#endif
     return sharedRegisterType(rawType, registeredInstance, options);
   },
 
@@ -235,30 +204,28 @@ var LibraryEmbind = {
     registerType(rawType, {
       isVoid: true, // void return values can be optimized out sometimes
       name,
-      argPackAdvance: 0,
-      'fromWireType': () => undefined,
+      fromWireType: () => undefined,
       // TODO: assert if anything else is given?
-      'toWireType': (destructors, o) => undefined,
+      toWireType: (destructors, o) => undefined,
     });
   },
 
   _embind_register_bool__docs: '/** @suppress {globalThis} */',
-  _embind_register_bool__deps: ['$AsciiToString', '$registerType', '$GenericWireTypeSize'],
+  _embind_register_bool__deps: ['$AsciiToString', '$registerType'],
   _embind_register_bool: (rawType, name, trueValue, falseValue) => {
     name = AsciiToString(name);
     registerType(rawType, {
       name,
-      'fromWireType': function(wt) {
+      fromWireType: function(wt) {
         // ambiguous emscripten ABI: sometimes return values are
         // true or false, and sometimes integers (0 or 1)
         return !!wt;
       },
-      'toWireType': function(destructors, o) {
+      toWireType: function(destructors, o) {
         return o ? trueValue : falseValue;
       },
-      argPackAdvance: GenericWireTypeSize,
-      'readValueFromPointer': function(pointer) {
-        return this['fromWireType'](HEAPU8[pointer]);
+      readValueFromPointer: function(pointer) {
+        return this.fromWireType(HEAPU8[pointer]);
       },
       destructorFunction: null, // This type does not need a destructor
     });
@@ -291,14 +258,14 @@ var LibraryEmbind = {
   $enumReadValueFromPointer: (name, width, signed) => {
     switch (width) {
       case 1: return signed ?
-        function(pointer) { return this['fromWireType']({{{ makeGetValue('pointer', 0, 'i8') }}}) } :
-        function(pointer) { return this['fromWireType']({{{ makeGetValue('pointer', 0, 'u8') }}}) };
+        function(pointer) { return this.fromWireType({{{ makeGetValue('pointer', 0, 'i8') }}}) } :
+        function(pointer) { return this.fromWireType({{{ makeGetValue('pointer', 0, 'u8') }}}) };
       case 2: return signed ?
-        function(pointer) { return this['fromWireType']({{{ makeGetValue('pointer', 0, 'i16') }}}) } :
-        function(pointer) { return this['fromWireType']({{{ makeGetValue('pointer', 0, 'u16') }}}) };
+        function(pointer) { return this.fromWireType({{{ makeGetValue('pointer', 0, 'i16') }}}) } :
+        function(pointer) { return this.fromWireType({{{ makeGetValue('pointer', 0, 'u16') }}}) };
       case 4: return signed ?
-        function(pointer) { return this['fromWireType']({{{ makeGetValue('pointer', 0, 'i32') }}}) } :
-        function(pointer) { return this['fromWireType']({{{ makeGetValue('pointer', 0, 'u32') }}}) };
+        function(pointer) { return this.fromWireType({{{ makeGetValue('pointer', 0, 'i32') }}}) } :
+        function(pointer) { return this.fromWireType({{{ makeGetValue('pointer', 0, 'u32') }}}) };
       default:
         throw new TypeError(`invalid integer width (${width}): ${name}`);
     }
@@ -308,10 +275,10 @@ var LibraryEmbind = {
   $floatReadValueFromPointer: (name, width) => {
     switch (width) {
       case 4: return function(pointer) {
-        return this['fromWireType']({{{ makeGetValue('pointer', 0, 'float') }}});
+        return this.fromWireType({{{ makeGetValue('pointer', 0, 'float') }}});
       };
       case 8: return function(pointer) {
-        return this['fromWireType']({{{ makeGetValue('pointer', 0, 'double') }}});
+        return this.fromWireType({{{ makeGetValue('pointer', 0, 'double') }}});
       };
       default:
         throw new TypeError(`invalid float width (${width}): ${name}`);
@@ -351,8 +318,8 @@ var LibraryEmbind = {
 
     registerType(primitiveType, {
       name,
-      'fromWireType': fromWireType,
-      'toWireType': (destructors, value) => {
+      fromWireType: fromWireType,
+      toWireType: (destructors, value) => {
 #if ASSERTIONS
         if (typeof value != "number" && typeof value != "boolean") {
           throw new TypeError(`Cannot convert "${embindRepr(value)}" to ${name}`);
@@ -363,8 +330,7 @@ var LibraryEmbind = {
         // https://www.w3.org/TR/wasm-js-api-1/#towebassemblyvalue
         return value;
       },
-      argPackAdvance: GenericWireTypeSize,
-      'readValueFromPointer': integerReadValueFromPointer(name, size, minRange !== 0),
+      readValueFromPointer: integerReadValueFromPointer(name, size, minRange !== 0),
       destructorFunction: null, // This type does not need a destructor
     });
   },
@@ -402,8 +368,8 @@ var LibraryEmbind = {
 
     registerType(primitiveType, {
       name,
-      'fromWireType': fromWireType,
-      'toWireType': (destructors, value) => {
+      fromWireType: fromWireType,
+      toWireType: (destructors, value) => {
         if (typeof value == "number") {
           value = BigInt(value);
         }
@@ -415,8 +381,7 @@ var LibraryEmbind = {
 #endif
         return value;
       },
-      argPackAdvance: GenericWireTypeSize,
-      'readValueFromPointer': integerReadValueFromPointer(name, size, !isUnsignedType),
+      readValueFromPointer: integerReadValueFromPointer(name, size, !isUnsignedType),
       destructorFunction: null, // This type does not need a destructor
     });
   },
@@ -435,8 +400,8 @@ var LibraryEmbind = {
     name = AsciiToString(name);
     registerType(rawType, {
       name,
-      'fromWireType': (value) => value,
-      'toWireType': (destructors, value) => {
+      fromWireType: (value) => value,
+      toWireType: (destructors, value) => {
 #if ASSERTIONS
         if (typeof value != "number" && typeof value != "boolean") {
           throw new TypeError(`Cannot convert ${embindRepr(value)} to ${this.name}`);
@@ -446,15 +411,14 @@ var LibraryEmbind = {
         // https://www.w3.org/TR/wasm-js-api-1/#towebassemblyvalue
         return value;
       },
-      argPackAdvance: GenericWireTypeSize,
-      'readValueFromPointer': floatReadValueFromPointer(name, size),
+      readValueFromPointer: floatReadValueFromPointer(name, size),
       destructorFunction: null, // This type does not need a destructor
     });
   },
 
   $readPointer__docs: '/** @suppress {globalThis} */',
   $readPointer: function(pointer) {
-    return this['fromWireType']({{{ makeGetValue('pointer', '0', '*') }}});
+    return this.fromWireType({{{ makeGetValue('pointer', '0', '*') }}});
   },
 
   _embind_register_std_string__deps: [
@@ -463,52 +427,31 @@ var LibraryEmbind = {
     '$stringToUTF8', '$lengthBytesUTF8', 'malloc', 'free'],
   _embind_register_std_string: (rawType, name) => {
     name = AsciiToString(name);
-    var stdStringIsUTF8
-#if EMBIND_STD_STRING_IS_UTF8
-    = true;
-#else
-    = false;
-#endif
+    var stdStringIsUTF8 = {{{ EMBIND_STD_STRING_IS_UTF8 }}};
 
     registerType(rawType, {
       name,
       // For some method names we use string keys here since they are part of
       // the public/external API and/or used by the runtime-generated code.
-      'fromWireType'(value) {
-        var length = {{{ makeGetValue('value', '0', SIZE_TYPE) }}};
+      fromWireType(value) {
+        var length = {{{ makeGetValue('value', '0', '*') }}};
         var payload = value + {{{ POINTER_SIZE }}};
 
         var str;
         if (stdStringIsUTF8) {
-          var decodeStartPtr = payload;
-          // Looping here to support possible embedded '0' bytes
-          for (var i = 0; i <= length; ++i) {
-            var currentBytePtr = payload + i;
-            if (i == length || HEAPU8[currentBytePtr] == 0) {
-              var maxRead = currentBytePtr - decodeStartPtr;
-              var stringSegment = UTF8ToString(decodeStartPtr, maxRead);
-              if (str === undefined) {
-                str = stringSegment;
-              } else {
-                str += String.fromCharCode(0);
-                str += stringSegment;
-              }
-              decodeStartPtr = currentBytePtr + 1;
-            }
-          }
+          str = UTF8ToString(payload, length, true);
         } else {
-          var a = new Array(length);
+          str = '';
           for (var i = 0; i < length; ++i) {
-            a[i] = String.fromCharCode(HEAPU8[payload + i]);
+            str += String.fromCharCode(HEAPU8[payload + i]);
           }
-          str = a.join('');
         }
 
         _free(value);
 
         return str;
       },
-      'toWireType'(destructors, value) {
+      toWireType(destructors, value) {
         if (value instanceof ArrayBuffer) {
           value = new Uint8Array(value);
         }
@@ -552,8 +495,7 @@ var LibraryEmbind = {
         }
         return base;
       },
-      argPackAdvance: GenericWireTypeSize,
-      'readValueFromPointer': readPointer,
+      readValueFromPointer: readPointer,
       destructorFunction(ptr) {
         _free(ptr);
       },
@@ -567,47 +509,31 @@ var LibraryEmbind = {
     ],
   _embind_register_std_wstring: (rawType, charSize, name) => {
     name = AsciiToString(name);
-    var decodeString, encodeString, readCharAt, lengthBytesUTF;
+    var decodeString, encodeString, lengthBytesUTF;
     if (charSize === 2) {
       decodeString = UTF16ToString;
       encodeString = stringToUTF16;
       lengthBytesUTF = lengthBytesUTF16;
-      readCharAt = (pointer) => {{{ makeGetValue('pointer', 0, 'u16') }}};
-    } else if (charSize === 4) {
+    } else {
+#if ASSERTIONS
+      assert(charSize === 4, 'only 2-byte and 4-byte strings are currently supported');
+#endif
       decodeString = UTF32ToString;
       encodeString = stringToUTF32;
       lengthBytesUTF = lengthBytesUTF32;
-      readCharAt = (pointer) => {{{ makeGetValue('pointer', 0, 'u32') }}};
     }
     registerType(rawType, {
       name,
-      'fromWireType': (value) => {
+      fromWireType: (value) => {
         // Code mostly taken from _embind_register_std_string fromWireType
         var length = {{{ makeGetValue('value', 0, '*') }}};
-        var str;
-
-        var decodeStartPtr = value + {{{ POINTER_SIZE }}};
-        // Looping here to support possible embedded '0' bytes
-        for (var i = 0; i <= length; ++i) {
-          var currentBytePtr = value + {{{ POINTER_SIZE }}} + i * charSize;
-          if (i == length || readCharAt(currentBytePtr) == 0) {
-            var maxReadBytes = currentBytePtr - decodeStartPtr;
-            var stringSegment = decodeString(decodeStartPtr, maxReadBytes);
-            if (str === undefined) {
-              str = stringSegment;
-            } else {
-              str += String.fromCharCode(0);
-              str += stringSegment;
-            }
-            decodeStartPtr = currentBytePtr + charSize;
-          }
-        }
+        var str = decodeString(value + {{{ POINTER_SIZE }}}, length * charSize, true);
 
         _free(value);
 
         return str;
       },
-      'toWireType': (destructors, value) => {
+      toWireType: (destructors, value) => {
         if (!(typeof value == 'string')) {
           throwBindingError(`Cannot pass non-string to C++ string type ${name}`);
         }
@@ -624,8 +550,7 @@ var LibraryEmbind = {
         }
         return ptr;
       },
-      argPackAdvance: GenericWireTypeSize,
-      'readValueFromPointer': readPointer,
+      readValueFromPointer: readPointer,
       destructorFunction(ptr) {
         _free(ptr);
       }
@@ -641,6 +566,11 @@ var LibraryEmbind = {
     __embind_register_emval(rawType);
   },
 
+  _embind_register_user_type_definition__deps: ['_embind_register_emval'],
+  _embind_register_user_type_definition: (rawType, name, definition) => {
+    __embind_register_emval(rawType);
+  },
+
   _embind_register_optional__deps: ['$registerType', '$EmValOptionalType'],
   _embind_register_optional: (rawOptionalType, rawType) => {
     registerType(rawOptionalType, EmValOptionalType);
@@ -674,9 +604,8 @@ var LibraryEmbind = {
     name = AsciiToString(name);
     registerType(rawType, {
       name,
-      'fromWireType': decodeMemoryView,
-      argPackAdvance: GenericWireTypeSize,
-      'readValueFromPointer': decodeMemoryView,
+      fromWireType: decodeMemoryView,
+      readValueFromPointer: decodeMemoryView,
     }, {
       ignoreDuplicateRegistrations: true,
     });
@@ -742,7 +671,7 @@ var LibraryEmbind = {
     // TODO: Remove this completely once all function invokers are being dynamically generated.
     var needsDestructorStack = usesDestructorStack(argTypes);
 
-    var returns = (argTypes[0].name !== 'void');
+    var returns = !argTypes[0].isVoid;
 
     var expectedArgCount = argCount - 2;
 #if ASSERTIONS
@@ -764,11 +693,11 @@ var LibraryEmbind = {
       invokerFuncArgs.length = isClassMethodFunc ? 2 : 1;
       invokerFuncArgs[0] = cppTargetFunc;
       if (isClassMethodFunc) {
-        thisWired = argTypes[1]['toWireType'](destructors, this);
+        thisWired = argTypes[1].toWireType(destructors, this);
         invokerFuncArgs[1] = thisWired;
       }
       for (var i = 0; i < expectedArgCount; ++i) {
-        argsWired[i] = argTypes[i + 2]['toWireType'](destructors, args[i]);
+        argsWired[i] = argTypes[i + 2].toWireType(destructors, args[i]);
         invokerFuncArgs.push(argsWired[i]);
       }
 
@@ -791,7 +720,7 @@ var LibraryEmbind = {
   #endif
 
         if (returns) {
-          return argTypes[0]['fromWireType'](rv);
+          return argTypes[0].fromWireType(rv);
         }
       }
 
@@ -810,12 +739,15 @@ var LibraryEmbind = {
 #else
     // Builld the arguments that will be passed into the closure around the invoker
     // function.
-    var closureArgs = [humanName, throwBindingError, cppInvokerFunc, cppTargetFunc, runDestructors, argTypes[0], argTypes[1]];
+    var retType = argTypes[0];
+    var instType = argTypes[1];
+    var closureArgs = [humanName, throwBindingError, cppInvokerFunc, cppTargetFunc, runDestructors, retType.fromWireType.bind(retType), instType?.toWireType.bind(instType)];
 #if EMSCRIPTEN_TRACING
     closureArgs.push(Module);
 #endif
-    for (var i = 0; i < argCount - 2; ++i) {
-      closureArgs.push(argTypes[i+2]);
+    for (var i = 2; i < argCount; ++i) {
+      var argType = argTypes[i];
+      closureArgs.push(argType.toWireType.bind(argType));
     }
 #if ASYNCIFY == 1
     closureArgs.push(Asyncify);
@@ -837,8 +769,8 @@ var LibraryEmbind = {
     var invokerFn = InvokerFunctions[signature](...closureArgs);
 #else
 
-    let [args, invokerFnBody] = createJsInvoker(argTypes, isClassMethodFunc, returns, isAsync);
-    var invokerFn = new Function(...args, invokerFnBody)(...closureArgs);
+    let invokerFactory = createJsInvoker(argTypes, isClassMethodFunc, returns, isAsync);
+    var invokerFn = invokerFactory(...closureArgs);
 #endif
 #endif
     return createNamedFunction(humanName, invokerFn);
@@ -972,17 +904,17 @@ var LibraryEmbind = {
         var setterArgumentType = elementTypes[i + elementsLength];
         var setter = elt.setter;
         var setterContext = elt.setterContext;
-        elt.read = (ptr) => getterReturnType['fromWireType'](getter(getterContext, ptr));
+        elt.read = (ptr) => getterReturnType.fromWireType(getter(getterContext, ptr));
         elt.write = (ptr, o) => {
           var destructors = [];
-          setter(setterContext, ptr, setterArgumentType['toWireType'](destructors, o));
+          setter(setterContext, ptr, setterArgumentType.toWireType(destructors, o));
           runDestructors(destructors);
         };
       });
 
       return [{
         name: reg.name,
-        'fromWireType': (ptr) => {
+        fromWireType: (ptr) => {
           var rv = new Array(elementsLength);
           for (var i = 0; i < elementsLength; ++i) {
             rv[i] = elements[i].read(ptr);
@@ -990,7 +922,7 @@ var LibraryEmbind = {
           rawDestructor(ptr);
           return rv;
         },
-        'toWireType': (destructors, o) => {
+        toWireType: (destructors, o) => {
           if (elementsLength !== o.length) {
             throw new TypeError(`Incorrect number of tuple elements for ${reg.name}: expected=${elementsLength}, actual=${o.length}`);
           }
@@ -1003,8 +935,7 @@ var LibraryEmbind = {
           }
           return ptr;
         },
-        argPackAdvance: GenericWireTypeSize,
-        'readValueFromPointer': readPointer,
+        readValueFromPointer: readPointer,
         destructorFunction: rawDestructor,
       }];
     });
@@ -1077,10 +1008,10 @@ var LibraryEmbind = {
         var setter = field.setter;
         var setterContext = field.setterContext;
         fields[fieldName] = {
-          read: (ptr) => getterReturnType['fromWireType'](getter(getterContext, ptr)),
+          read: (ptr) => getterReturnType.fromWireType(getter(getterContext, ptr)),
           write: (ptr, o) => {
             var destructors = [];
-            setter(setterContext, ptr, setterArgumentType['toWireType'](destructors, o));
+            setter(setterContext, ptr, setterArgumentType.toWireType(destructors, o));
             runDestructors(destructors);
           },
           optional,
@@ -1089,7 +1020,7 @@ var LibraryEmbind = {
 
       return [{
         name: reg.name,
-        'fromWireType': (ptr) => {
+        fromWireType: (ptr) => {
           var rv = {};
           for (var i in fields) {
             rv[i] = fields[i].read(ptr);
@@ -1097,7 +1028,7 @@ var LibraryEmbind = {
           rawDestructor(ptr);
           return rv;
         },
-        'toWireType': (destructors, o) => {
+        toWireType: (destructors, o) => {
           // todo: Here we have an opportunity for -O3 level "unsafe" optimizations:
           // assume all fields are present without checking.
           for (var fieldName in fields) {
@@ -1114,8 +1045,7 @@ var LibraryEmbind = {
           }
           return ptr;
         },
-        argPackAdvance: GenericWireTypeSize,
-        'readValueFromPointer': readPointer,
+        readValueFromPointer: readPointer,
         destructorFunction: rawDestructor,
       }];
     });
@@ -1250,7 +1180,6 @@ var LibraryEmbind = {
     '$RegisteredPointer',
     '$readPointer',
     '$RegisteredPointer_fromWireType',
-    '$GenericWireTypeSize',
   ],
   $init_RegisteredPointer: () => {
     Object.assign(RegisteredPointer.prototype, {
@@ -1263,9 +1192,8 @@ var LibraryEmbind = {
       destructor(ptr) {
         this.rawDestructor?.(ptr);
       },
-      argPackAdvance: GenericWireTypeSize,
-      'readValueFromPointer': readPointer,
-      'fromWireType': RegisteredPointer_fromWireType,
+      readValueFromPointer: readPointer,
+      fromWireType: RegisteredPointer_fromWireType,
     });
   },
 
@@ -1312,14 +1240,14 @@ var LibraryEmbind = {
 
     if (!isSmartPointer && registeredClass.baseClass === undefined) {
       if (isConst) {
-        this['toWireType'] = constNoSmartPtrRawPointerToWireType;
+        this.toWireType = constNoSmartPtrRawPointerToWireType;
         this.destructorFunction = null;
       } else {
-        this['toWireType'] = nonConstNoSmartPtrRawPointerToWireType;
+        this.toWireType = nonConstNoSmartPtrRawPointerToWireType;
         this.destructorFunction = null;
       }
     } else {
-      this['toWireType'] = genericPointerToWireType;
+      this.toWireType = genericPointerToWireType;
       // Here we must leave this.destructorFunction undefined, since whether genericPointerToWireType returns
       // a pointer that needs to be freed up is runtime-dependent, and cannot be evaluated at registration time.
       // TODO: Create an alternative mechanism that allows removing the use of var destructors = []; array in
@@ -1446,7 +1374,7 @@ var LibraryEmbind = {
 #endif
   ],
   $attachFinalizer: (handle) => {
-    if ('undefined' === typeof FinalizationRegistry) {
+    if (!globalThis.FinalizationRegistry) {
       attachFinalizer = (handle) => handle;
       return handle;
     }
@@ -2000,7 +1928,7 @@ var LibraryEmbind = {
         var desc = {
           get() {
             var ptr = validateThis(this, classType, humanName + ' getter');
-            return getterReturnType['fromWireType'](getter(getterContext, ptr));
+            return getterReturnType.fromWireType(getter(getterContext, ptr));
           },
           enumerable: true
         };
@@ -2011,7 +1939,7 @@ var LibraryEmbind = {
           desc.set = function(v) {
             var ptr = validateThis(this, classType, humanName + ' setter');
             var destructors = [];
-            setter(setterContext, ptr, setterArgumentType['toWireType'](destructors, v));
+            setter(setterContext, ptr, setterArgumentType.toWireType(destructors, v));
             runDestructors(destructors);
           };
         }
@@ -2134,7 +2062,7 @@ var LibraryEmbind = {
         fieldType = fieldType[0];
         var desc = {
           get() {
-            return fieldType['fromWireType'](getter(rawFieldPtr));
+            return fieldType.fromWireType(getter(rawFieldPtr));
           },
           enumerable: true
         };
@@ -2143,7 +2071,7 @@ var LibraryEmbind = {
           setter = embind__requireFunction(setterSignature, setter);
           desc.set = (v) => {
             var destructors = [];
-            setter(rawFieldPtr, fieldType['toWireType'](destructors, v));
+            setter(rawFieldPtr, fieldType.toWireType(destructors, v));
             runDestructors(destructors);
           };
         }
@@ -2283,12 +2211,11 @@ var LibraryEmbind = {
     registerType(rawType, {
       name,
       constructor: ctor,
-      'fromWireType': function(c) {
+      fromWireType: function(c) {
         return this.constructor.values[c];
       },
-      'toWireType': (destructors, c) => c.value,
-      argPackAdvance: GenericWireTypeSize,
-      'readValueFromPointer': enumReadValueFromPointer(name, size, isSigned),
+      toWireType: (destructors, c) => c.value,
+      readValueFromPointer: enumReadValueFromPointer(name, size, isSigned),
       destructorFunction: null,
     });
     exposePublicSymbol(name, ctor);
@@ -2314,7 +2241,7 @@ var LibraryEmbind = {
     name = AsciiToString(name);
     whenDependentTypesAreResolved([], [type], (type) => {
       type = type[0];
-      Module[name] = type['fromWireType'](value);
+      Module[name] = type.fromWireType(value);
       return [];
     });
   },
