@@ -16,8 +16,8 @@
 // underscore.
 var WASM_EXPORTS = [];
 
-// Similar to above but only includes the global/data symbols.
-var WASM_GLOBAL_EXPORTS = [];
+// Similar to above but only includes the data symbols (address exports).
+var DATA_EXPORTS = [];
 
 // An array of all symbols exported from all the side modules specified on the
 // command line.
@@ -158,9 +158,6 @@ var MINIFY_WASM_IMPORTED_MODULES = false;
 // Whether to minify exports from the Wasm module.
 var MINIFY_WASM_EXPORT_NAMES = true;
 
-// Used to track whether target environment supports the 'globalThis' attribute.
-var SUPPORTS_GLOBALTHIS = false;
-
 // Used to track whether target environment supports the 'Promise.any'.
 var SUPPORTS_PROMISE_ANY = false;
 
@@ -183,7 +180,11 @@ var CAN_ADDRESS_2GB = false;
 // This has no effect if DWARF is not being emitted.
 var SEPARATE_DWARF = false;
 
-// New WebAssembly exception handling
+// Target WebAssembly exception handling instead of JavaScript-side exception
+// handling. Furthermore, if WASM_LEGACY_EXCEPTIONS=1, then old legacy Wasm
+// exception handling is used, and if WASM_LEGACY_EXCEPTIONS=0, then Wasm
+// exception handling is targeted.
+// Enabled by passing -fwasm-exceptions on the command line.
 var WASM_EXCEPTIONS = false;
 
 // Set to true if the program has a main function.  By default this is
@@ -191,10 +192,6 @@ var WASM_EXCEPTIONS = false;
 // EXPORTED_FUNCTIONS then this gets set to 0.
 var EXPECT_MAIN = true;
 
-// Return a "ready" Promise from the MODULARIZE factory function.
-// We disable this under some circumstance if we know its not needed.
-var USE_READY_PROMISE = true;
-
 // If true, building against Emscripten's wasm heap memory profiler.
 var MEMORYPROFILER = false;
 
@@ -257,8 +254,6 @@ var POST_JS_FILES = [];
 // Set when -pthread / -sPTHREADS is passed
 var PTHREADS = false;
 
-var BULK_MEMORY = false;
-
 var MINIFY_WHITESPACE = true;
 
 var ASYNCIFY_IMPORTS_EXCEPT_JS_LIBS = [];
@@ -276,3 +271,9 @@ var INCLUDE_WEBGL1_FALLBACK = true;
 var MINIFICATION_MAP = '';
 
 var OUTPUT_FORMAT = '';
+
+// Whether we should load the WASM source map at runtime.
+// This is enabled automatically when using -gsource-map with sanitizers.
+var LOAD_SOURCE_MAP = false;
+
+var ALIASES = [];
