@@ -3,8 +3,6 @@
 # University of Illinois/NCSA Open Source License.  Both these licenses can be
 # found in the LICENSE file.
 
-from tools.toolchain_profiler import ToolchainProfiler
-
 import json
 import logging
 import os
@@ -14,11 +12,20 @@ import sys
 from enum import Enum, auto, unique
 from subprocess import PIPE
 
-from tools import shared, utils, ports, diagnostics, config
-from tools import cache, feature_matrix, colored_logger
+from tools import (
+  cache,
+  colored_logger,
+  config,
+  diagnostics,
+  feature_matrix,
+  ports,
+  shared,
+  utils,
+)
+from tools.settings import MEM_SIZE_SETTINGS, settings, user_settings
 from tools.shared import exit_with_error
-from tools.settings import settings, user_settings, MEM_SIZE_SETTINGS
-from tools.utils import removeprefix, read_file
+from tools.toolchain_profiler import ToolchainProfiler
+from tools.utils import read_file, removeprefix
 
 SIMD_INTEL_FEATURE_TOWER = ['-msse', '-msse2', '-msse3', '-mssse3', '-msse4.1', '-msse4.2', '-msse4', '-mavx', '-mavx2']
 SIMD_NEON_FLAGS = ['-mfpu=neon']
@@ -61,7 +68,6 @@ class EmccOptions:
     self.post_link = False
     self.save_temps = False
     self.executable = False
-    self.compiler_wrapper = None
     self.oformat = None
     self.requested_debug = None
     self.emit_symbol_map = False
@@ -78,7 +84,6 @@ class EmccOptions:
     self.shell_path = None
     self.source_map_base = ''
     self.emit_tsd = ''
-    self.embind_emit_tsd = ''
     self.emrun = False
     self.cpu_profiler = False
     self.memory_profiler = False
@@ -107,6 +112,7 @@ class EmccOptions:
     self.sanitize_minimal_runtime = False
     self.sanitize = set()
     self.lib_dirs = []
+    self.fast_math = False
 
 
 def is_int(s):
@@ -296,9 +302,8 @@ def parse_args(newargs):  # noqa: C901, PLR0912, PLR0915
         settings.SHRINK_LEVEL = 0
         settings.DEBUG_LEVEL = max(settings.DEBUG_LEVEL, 1)
       elif requested_level == 'fast':
-        # TODO(https://github.com/emscripten-core/emscripten/issues/21497):
-        # If we ever map `-ffast-math` to `wasm-opt --fast-math` then
-        # then we should enable that too here.
+        # -Ofast typically includes -ffast-math semantics
+        options.fast_math = True
         requested_level = 3
         settings.SHRINK_LEVEL = 0
       else:
@@ -368,7 +373,8 @@ def parse_args(newargs):  # noqa: C901, PLR0912, PLR0915
           settings.EMIT_NAME_SECTION = 1
         # if we don't need to preserve LLVM debug info, do not keep this flag
         # for clang
-        if settings.DEBUG_LEVEL < 3:
+        if (settings.DEBUG_LEVEL < 3 and not
+            (settings.GENERATE_SOURCE_MAP or settings.SEPARATE_DWARF)):
           newargs[i] = '-g0'
         else:
           # for 3+, report -g3 to clang as -g4 etc. are not accepted
@@ -394,9 +400,9 @@ def parse_args(newargs):  # noqa: C901, PLR0912, PLR0915
           else:
             settings.SEPARATE_DWARF = True
           settings.GENERATE_DWARF = 1
+          settings.DEBUG_LEVEL = 3
         elif requested_level in ['source-map', 'source-map=inline']:
           settings.GENERATE_SOURCE_MAP = 1 if requested_level == 'source-map' else 2
-          settings.EMIT_NAME_SECTION = 1
           newargs[i] = '-g'
         elif requested_level == 'z':
           # Ignore `-gz`.  We don't support debug info compression.
@@ -407,10 +413,7 @@ def parse_args(newargs):  # noqa: C901, PLR0912, PLR0915
           # clang and make the emscripten code treat it like any other DWARF.
           settings.GENERATE_DWARF = 1
           settings.EMIT_NAME_SECTION = 1
-        # In all cases set the emscripten debug level to 3 so that we do not
-        # strip during link (during compile, this does not make a difference).
-        # TODO: possibly decouple some of these flags from the final debug level (#20462)
-        settings.DEBUG_LEVEL = 3
+          settings.DEBUG_LEVEL = 3
     elif check_flag('-profiling') or check_flag('--profiling'):
       settings.DEBUG_LEVEL = max(settings.DEBUG_LEVEL, 2)
       settings.EMIT_NAME_SECTION = 1
@@ -514,22 +517,18 @@ def parse_args(newargs):  # noqa: C901, PLR0912, PLR0915
     elif check_flag('--threadprofiler'):
       settings_changes.append('PTHREADS_PROFILING=1')
     elif arg in ('-fcolor-diagnostics', '-fdiagnostics-color', '-fdiagnostics-color=always'):
-      diagnostics.color_enabled = True
-    elif arg in ('-fno-color-diagnostics', '-fdiagnostics-color=never'):
-      diagnostics.color_enabled = False
-    elif arg == '-fansi-escape-codes':
-      diagnostics.force_ansi = True
+      colored_logger.enable(force=True)
+    elif arg in ('-fno-color-diagnostics', '-fno-diagnostics-color', '-fdiagnostics-color=never'):
+      colored_logger.disable()
     elif arg == '-fno-exceptions':
       settings.DISABLE_EXCEPTION_CATCHING = 1
       settings.DISABLE_EXCEPTION_THROWING = 1
       settings.WASM_EXCEPTIONS = 0
     elif arg == '-mbulk-memory':
-      settings.BULK_MEMORY = 1
       feature_matrix.enable_feature(feature_matrix.Feature.BULK_MEMORY,
                                     '-mbulk-memory',
                                     override=True)
     elif arg == '-mno-bulk-memory':
-      settings.BULK_MEMORY = 0
       feature_matrix.disable_feature(feature_matrix.Feature.BULK_MEMORY)
     elif arg == '-msign-ext':
       feature_matrix.enable_feature(feature_matrix.Feature.SIGN_EXT,
@@ -553,6 +552,8 @@ def parse_args(newargs):  # noqa: C901, PLR0912, PLR0915
       settings.WASM_EXCEPTIONS = 1
     elif arg == '-fignore-exceptions':
       settings.DISABLE_EXCEPTION_CATCHING = 1
+    elif arg == '-ffast-math':
+      options.fast_math = True
     elif check_arg('--default-obj-ext'):
       exit_with_error('--default-obj-ext is no longer supported by emcc')
     elif arg.startswith('-fsanitize=cfi'):
@@ -576,9 +577,6 @@ def parse_args(newargs):  # noqa: C901, PLR0912, PLR0915
       settings.USE_PTHREADS = 0
     elif arg == '-pthreads':
       exit_with_error('unrecognized command-line option `-pthreads`; did you mean `-pthread`?')
-    elif arg in ('-fno-diagnostics-color', '-fdiagnostics-color=never'):
-      colored_logger.disable()
-      diagnostics.color_enabled = False
     elif arg == '-fno-rtti':
       settings.USE_RTTI = 0
     elif arg == '-frtti':
