@@ -3,25 +3,22 @@
 # University of Illinois/NCSA Open Source License.  Both these licenses can be
 # found in the LICENSE file.
 
-import re
-from time import time
-from .toolchain_profiler import ToolchainProfiler
-
 import itertools
 import logging
 import os
-import shutil
-import textwrap
+import re
 import shlex
+import shutil
 import subprocess
+import textwrap
 from enum import IntEnum, auto
 from glob import iglob
+from time import time
 from typing import List, Optional
 
-from . import shared, building, utils
-from . import diagnostics
-from . import cache
+from . import building, cache, diagnostics, shared, utils
 from .settings import settings
+from .toolchain_profiler import ToolchainProfiler
 from .utils import read_file
 
 logger = logging.getLogger('system_libs')
@@ -70,7 +67,7 @@ def get_base_cflags(build_dir, force_object_files=False, preprocess=True):
     if preprocess:
       flags += ['-DEMSCRIPTEN_DYNAMIC_LINKING']
   if settings.MEMORY64:
-    flags += ['-Wno-experimental', '-sMEMORY64=' + str(settings.MEMORY64)]
+    flags += ['-sMEMORY64=' + str(settings.MEMORY64)]
 
   source_dir = utils.path_from_root()
   relative_source_dir = os.path.relpath(source_dir, build_dir)
@@ -140,7 +137,7 @@ def objectfile_sort_key(filename):
 
 def create_lib(libname, inputs):
   """Create a library from a set of input objects."""
-  suffix = shared.suffix(libname)
+  suffix = utils.suffix(libname)
 
   inputs = sorted(inputs, key=objectfile_sort_key)
   if suffix in ('.bc', '.o'):
@@ -229,13 +226,13 @@ rule archive
   description = AR $out
 
 '''
-  suffix = shared.suffix(libname)
+  suffix = utils.suffix(libname)
   build_dir = os.path.dirname(filename)
 
   if suffix == '.o':
     assert len(input_files) == 1
     input_file = escape_ninja_path(input_files[0])
-    depfile = shared.unsuffixed_basename(input_file) + '.d'
+    depfile = utils.unsuffixed_basename(input_file) + '.d'
     out += f'build {escape_ninja_path(libname)}: direct_cc {input_file}\n'
     out += f'  with_depfile = {depfile}\n'
   else:
@@ -245,7 +242,7 @@ rule archive
       # insensitive filesystem to handle, for example, _exit.o and _Exit.o.
       # This is done even on case sensitive filesystem so that builds are
       # reproducible across platforms.
-      object_basename = shared.unsuffixed_basename(src).lower()
+      object_basename = utils.unsuffixed_basename(src).lower()
       o = os.path.join(build_dir, object_basename + '.o')
       object_uuid = 0
       # Find a unique basename
@@ -253,7 +250,7 @@ rule archive
         object_uuid += 1
         o = os.path.join(build_dir, f'{object_basename}__{object_uuid}.o')
       objects.append(o)
-      ext = shared.suffix(src)
+      ext = utils.suffix(src)
       if ext == '.s':
         cmd = 'asm'
         flags = asflags
@@ -447,7 +444,7 @@ class Library:
     if self.get_ext() != '.a':
       return fullpath
     # For libraries (.a) files, we pass the abbreviated `-l` form.
-    base = shared.unsuffixed_basename(fullpath)
+    base = utils.unsuffixed_basename(fullpath)
     return '-l' + utils.removeprefix(base, 'lib')
 
   def get_files(self):
@@ -493,7 +490,7 @@ class Library:
     objects = set()
     cflags = self.get_cflags()
     for src in self.get_files():
-      ext = shared.suffix(src)
+      ext = utils.suffix(src)
       if ext in {'.s', '.S', '.c'}:
         cmd = shared.EMCC
       else:
@@ -508,7 +505,7 @@ class Library:
         cmd += cflags
       cmd = self.customize_build_cmd(cmd, src)
 
-      object_basename = shared.unsuffixed_basename(src).lower()
+      object_basename = utils.unsuffixed_basename(src).lower()
       o = os.path.join(build_dir, object_basename + '.o')
       if o in objects:
         # If we have seen a file with the same name before, we need a separate
@@ -529,7 +526,7 @@ class Library:
         src = utils.normalize_path(src)
         batches.setdefault(tuple(cmd), []).append(src)
         # No -o in command, use original file name.
-        o = os.path.join(build_dir, shared.unsuffixed_basename(src) + '.o')
+        o = os.path.join(build_dir, utils.unsuffixed_basename(src) + '.o')
       else:
         commands.append(cmd + [src, '-o', o])
       objects.add(o)
@@ -987,6 +984,7 @@ class libcompiler_rt(MTLibrary, SjLjLibrary):
         'emscripten_setjmp.c',
         'emscripten_exception_builtins.c',
         'emscripten_tempret.s',
+        '__c_longjmp.S',
         '__trap.c',
       ])
 
@@ -1640,8 +1638,6 @@ class libcxxabi(ExceptionLibrary, MTLibrary, DebugLibrary):
       # The code used to interpret exceptions during terminate
       # is not compatible with emscripten exceptions.
       cflags.append('-DLIBCXXABI_SILENT_TERMINATE')
-    elif self.eh_mode == Exceptions.WASM_LEGACY:
-      cflags.append('-D__WASM_EXCEPTIONS__')
     return cflags
 
   def get_files(self):
@@ -1661,6 +1657,7 @@ class libcxxabi(ExceptionLibrary, MTLibrary, DebugLibrary):
       'stdlib_typeinfo.cpp',
       'private_typeinfo.cpp',
       'cxa_exception_js_utils.cpp',
+      '__cpp_exception.S',
     ]
     if self.eh_mode == Exceptions.NONE:
       filenames += ['cxa_noexception.cpp']
@@ -1719,12 +1716,6 @@ class libcxx(ExceptionLibrary, MTLibrary, DebugLibrary):
     'tzdb_list.cpp',
   ]
 
-  def get_cflags(self):
-    cflags = super().get_cflags()
-    if self.eh_mode in (Exceptions.WASM_LEGACY, Exceptions.WASM):
-      cflags.append('-D__WASM_EXCEPTIONS__')
-    return cflags
-
 
 class libunwind(ExceptionLibrary, MTLibrary):
   name = 'libunwind'
@@ -1755,8 +1746,6 @@ class libunwind(ExceptionLibrary, MTLibrary):
       cflags.append('-D_LIBUNWIND_HAS_NO_EXCEPTIONS')
     elif self.eh_mode == Exceptions.EMSCRIPTEN:
       cflags.append('-D__EMSCRIPTEN_EXCEPTIONS__')
-    elif self.eh_mode in (Exceptions.WASM_LEGACY, Exceptions.WASM):
-      cflags.append('-D__WASM_EXCEPTIONS__')
     return cflags
 
 
@@ -1955,21 +1944,6 @@ class libGL(MTLibrary):
     )
 
 
-class libwebgpu(MTLibrary):
-  name = 'libwebgpu'
-
-  src_dir = 'system/lib/webgpu'
-  src_files = ['webgpu.cpp']
-
-
-class libwebgpu_cpp(MTLibrary):
-  name = 'libwebgpu_cpp'
-
-  cflags = ['-std=c++11']
-  src_dir = 'system/lib/webgpu'
-  src_files = ['webgpu_cpp.cpp']
-
-
 class libembind(MTLibrary):
   name = 'libembind'
   never_force = True
@@ -2433,11 +2407,6 @@ def get_libs_to_link(options):
   else:
     add_library('libsockets')
 
-  if settings.USE_WEBGPU:
-    add_library('libwebgpu')
-    if settings.LINK_AS_CXX:
-      add_library('libwebgpu_cpp')
-
   if settings.WASM_WORKERS and (not settings.SINGLE_FILE and
                                 not settings.RELOCATABLE and
                                 not settings.PROXY_TO_WORKER):
@@ -2504,7 +2473,7 @@ def safe_copytree(src, dst, excludes=None):
     if entry.is_dir():
       safe_copytree(srcname, dstname, excludes)
     else:
-      shared.safe_copy(srcname, dstname)
+      utils.safe_copy(srcname, dstname)
 
 
 def install_system_headers(stamp):
